#ifndef XCORE_H
#define XCORE_H

#include <QtCore>
#include <cstdlib>
#include <ctime>
#include <new>
#include "xcsv.h"
#include "xresult.h"

using namespace std;

typedef QList<int> XRow;
typedef QList<XRow> XMatrix;
typedef QPair<QString, int> XPair;
typedef QMap<XPair, int> XMap;

class XCore : public QThread
{
	Q_OBJECT
public:
	explicit XCore(const XCsv *csv, const XCsv *src = NULL, const int &iterations = 1, const int &seed = 0, QObject *parent = 0);

protected:
	void run();

private:
	bool _stop;
	int _iterations;
	const XCsv *_csv;
	const XCsv *_src;
	bool readCsv(const XCsv *csv, const QStringList &header, XMatrix &matrix, XMap &elements) const;
	bool csortMatrix(XMatrix &matrix, const int &column) const;
	bool iterate(const XMatrix &csvMatrix, const XMap &elements, const XMap &srcElements, const QStringList &header, XMatrix &outMatrix, XMap &outElements, QList<int> &outRows, const int &subsetSize) const;

signals:
	void percentChanged(int) const;
	void error(QString) const;
	void completed(XResult *) const;

public slots:
	void stopIterating();
};

#endif // XCORE_H
