#include "xcsv.h"

XCsv::XCsv(const QString &input, QObject *parent) :
		XFile(parent)
{
	QString data = this->getData(input);

	if (data != "")
		this->fromString(data);
}

XCsv::XCsv(const XCsvData &data, QObject *parent)
{
	_data = data;
	_columns = data.at(0).length();
}

const QVariant &XCsv::at(const int &row, const int &column) const
{
	return _data.at(row).at(column);
}

const XCsvRow &XCsv::at(const int &row) const
{
	return _data.at(row);
}

const XCsvData &XCsv::data() const
{
	return _data;
}

int XCsv::rows() const
{
	return _data.length();
}

int XCsv::columns() const
{
	return _columns;
}

bool XCsv::setCell(const int &row, const int &column, const QVariant &data)
{
	if (row >= _data.length())
		return false;

	if (column >= _columns)
		return false;

	_data[row][column] = data;
	return true;
}

bool XCsv::addRow(const int &index, const QString &header, const QVariant &fill)
{
	if (index < 0 || index > _data.length())
		return false;

	XCsvRow row;

	row << header;

	for (int i = 1; i < _columns; ++i)
	{
		row << fill;
	}

	if (index == _data.length())
		_data << row;
	else
		_data.insert(index, row);

	return true;
}

bool XCsv::removeRow(const int &index)
{
	if (index < 0 || index >= _data.length())
		return false;

	_data.removeAt(index);
	return true;
}

bool XCsv::addColumn(const int &index, const QString &header, const QVariant &fill)
{
	if (index < 0 || index > _columns)
		return false;

	if (index == _columns)
		_data[0] << header;
	else
		_data[0].insert(index, header);

	for (int i = 1; i < _data.length(); ++i)
	{
		if (index == _columns)
			_data[i] << fill;
		else
			_data[i].insert(index, fill);
	}

	_columns += 1;
	return true;
}

bool XCsv::removeColumn(const int &index)
{
	if (index < 0 || index >= _columns)
		return false;

	for (int i = 0; i < _data.length(); ++i)
	{
		_data[i].removeAt(index);
	}

	_columns -= 1;
	return true;
}

QString XCsv::toString() const
{
	QString str;
	QTextStream stream(&str);

	foreach (XCsvRow row, _data)
	{
		int count = 0;

		foreach (QVariant cell, row)
		{
			if (cell.type() == QVariant::String)
				stream << '"' << cell.toString() << '"';
			else
				stream << cell.toString();

			if (count++ < _columns - 1)
				stream << ',';
		}

		stream << endl;
	}

	return str;
}

void XCsv::fromString(QString &input)
{
	_columns = 0;

	QTextStream stream(&input);

	while (!stream.atEnd())
	{
		QString line = stream.readLine().trimmed();
		QStringList split = line.split(',');

		if (!_columns && split.length() > 1)
			_columns = split.length();
		else if (_columns && (split.length() != _columns))
			continue;

		XCsvRow row;

		foreach (QString str, split)
		{
			bool ok;
			int value = str.toInt(&ok);

			if (ok)
				row << value;
			else
			{
				QString temp = str.trimmed();

				if (temp.startsWith('"') && temp.endsWith('"'))
					temp = temp.mid(1, temp.length() - 2);

				row << temp;
			}
		}

		_data << row;
	}
}
