#ifndef XCSVMODEL_H
#define XCSVMODEL_H

#include <QtCore>
#include <QBrush>
#include "xcsv.h"

typedef QPair<int, int> XCsvError;

class XCsvTableModel : public QAbstractTableModel
{
	Q_OBJECT
public:
	explicit XCsvTableModel(XCsv *csv, const bool &readOnly = true, QObject *parent = 0);
	const QList<XCsvError> &errors() const;
	void check();
	bool isError(const QVariant &value) const;
	bool hasErrors() const;
	int rowCount(const QModelIndex &parent = QModelIndex()) const;
	int columnCount(const QModelIndex &parent = QModelIndex()) const;
	QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
	QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
	Qt::ItemFlags flags(const QModelIndex &index) const;
	bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole);
	bool setHeaderData(int section, Qt::Orientation orientation, const QVariant &value, int role = Qt::EditRole);
	bool insertRows(int row, int count, const QModelIndex &parent = QModelIndex());
	bool removeRows(int row, int count, const QModelIndex &parent = QModelIndex());
	bool insertColumns(int column, int count, const QModelIndex &parent = QModelIndex());
	bool removeColumns(int column, int count, const QModelIndex &parent = QModelIndex());

private:
	XCsv *_csv;
	bool _readOnly;
	QList<XCsvError> _errors;
	void checkCell(const int &row, const int &column);

signals:
	void modified();
};

#endif // XCSVMODEL_H
