#include "xgraphwidget.h"

XGraphWidget::XGraphWidget(QWidget *parent) :
		QwtPlot(parent)
{
	this->setAxisTitle(xBottom, "Size of the core (no. of accessions)");
	this->setAxisTitle(yLeft, "Number of alleles/variants");
	this->setCanvasBackground(Qt::white);

	_linePen = QPen(Qt::blue);
	_dotsPen = QPen(QBrush(Qt::blue), 5, Qt::SolidLine, Qt::RoundCap);
	_dummyPen = QPen(QBrush(Qt::white), 0);
	_horMarkerPen = QPen(Qt::red, 1, Qt::DotLine);
	_vertMarkerPen = QPen(Qt::darkGreen, 1, Qt::DotLine);
	_line = NULL;
	_dots = NULL;
	_dummy = NULL;
}

XGraphWidget::~XGraphWidget()
{
	delete _line;
	delete _dots;
	delete _dummy;

	foreach (QwtPlotMarker *marker, _horMarkers)
	{
		delete marker;
	}

	foreach (QwtPlotMarker *marker, _vertMarkers)
	{
		delete marker;
	}
}

void XGraphWidget::setResult(XResult *result)
{
	if (_line)
	{
		_line->detach();
		delete _line;
		_line = NULL;
	}

	if (_dots)
	{
		_dots->detach();
		delete _dots;
		_dots = NULL;
	}

	foreach (QwtPlotMarker *marker, _horMarkers)
	{
		marker->detach();
		delete marker;
	}

	_horMarkers.clear();

	foreach (QwtPlotMarker *marker, _vertMarkers)
	{
		marker->detach();
		delete marker;
	}

	_vertMarkers.clear();

	QVector<double> xData;
	QVector<double> yData;

	XCsv *csv = NULL;

	for (int i = 1; i <= 100; ++i)
	{
		if (result->csv(i) != csv)
		{
			csv = result->csv(i);

			int x = csv->rows() - 1;
			int y = result->count(i);
			QwtPlotMarker *marker = new QwtPlotMarker;

			marker->setLabel(QString::number(x));
			marker->setLabelAlignment(Qt::AlignRight | Qt::AlignBottom);
			marker->setLineStyle(QwtPlotMarker::VLine);
			marker->setLinePen(_vertMarkerPen);
			marker->setXValue(csv->rows() - 1);
			marker->attach(this);

			_vertMarkers << marker;
			xData << x;
			yData << y;
		}
	}

	_line = new QwtPlotCurve;
	_line->setStyle(QwtPlotCurve::Lines);
	_line->setRenderHint(QwtPlotItem::RenderAntialiased);
	_line->setPen(_linePen);
	_line->setData(xData, yData);
	_line->attach(this);

	_dots = new QwtPlotCurve;
	_dots->setStyle(QwtPlotCurve::Dots);
	_dots->setRenderHint(QwtPlotItem::RenderAntialiased);
	_dots->setPen(_dotsPen);
	_dots->setData(xData, yData);
	_dots->attach(this);

	xData.clear();
	xData << 0;
	yData.clear();
	yData << 0;

	_dummy = new QwtPlotCurve;
	_dummy->setStyle(QwtPlotCurve::Dots);
	_dummy->setPen(_dummyPen);
	_dummy->setData(xData, yData);
	_dummy->attach(this);

	double yFactor = _line->maxYValue() / 100.0;

	for (int i = 10; i <= 100; i += 10)
	{
		QwtPlotMarker *marker = new QwtPlotMarker;

		marker->setLabel(QString::number(i) + "%");
		marker->setLabelAlignment(Qt::AlignRight | Qt::AlignBottom);
		marker->setLineStyle(QwtPlotMarker::HLine);
		marker->setLinePen(_horMarkerPen);
		marker->setYValue(yFactor * i);
		marker->attach(this);

		_horMarkers << marker;
	}

	replot();
}
