#include "xmainwindow.h"

XMainWindow::XMainWindow(QWidget *parent) :
		QMainWindow(parent)
{
	_csv = NULL;
	_src = NULL;
	_workspace = NULL;

	_menuBar = new QMenuBar;
	_newAction = new QAction(tr("New..."), this);
	_saveAction = new QAction(tr("Save..."), this);
	_saveAsAction = new QAction(tr("Save As..."), this);
	_importAction = new QAction(tr("Import Workspace..."), this);
	_exportAction = new QAction(tr("Export Workspace..."), this);
	_quitAction = new QAction(tr("Quit"), this);
	_aboutAction = new QAction(tr("About"), this);
	_qtAction = new QAction(tr("About Qt"), this);
	_aboutBox = new QMessageBox(QMessageBox::Information, tr("About"), ABOUT_TEXT);

	_exportAction->setEnabled(false);
	_saveAction->setEnabled(false);
	_saveAsAction->setEnabled(false);

	_newAction->setShortcut(QKeySequence::New);
	_saveAction->setShortcut(QKeySequence::Save);
	_saveAsAction->setShortcut(QKeySequence::SaveAs);
	_quitAction->setShortcut(QKeySequence::Quit);

	this->connect(_newAction, SIGNAL(triggered()), this, SLOT(wizard()));
	this->connect(_saveAction, SIGNAL(triggered()), this, SLOT(save()));
	this->connect(_saveAsAction, SIGNAL(triggered()), this, SLOT(saveAs()));
	this->connect(_importAction, SIGNAL(triggered()), this, SLOT(importWorkspace()));
	this->connect(_exportAction, SIGNAL(triggered()), this, SLOT(exportWorkspace()));
	this->connect(_quitAction, SIGNAL(triggered()), this, SLOT(close()));
	this->connect(_aboutAction, SIGNAL(triggered()), _aboutBox, SLOT(exec()));
	this->connect(_qtAction, SIGNAL(triggered()), qApp, SLOT(aboutQt()));

	QMenu *fileMenu = new QMenu(tr("File"));

	fileMenu->addAction(_newAction);
	fileMenu->addAction(_saveAction);
	fileMenu->addAction(_saveAsAction);
	fileMenu->addAction(_importAction);
	fileMenu->addAction(_exportAction);
	fileMenu->addAction(_quitAction);

	QMenu *helpMenu = new QMenu(tr("Help"));

	helpMenu->addAction(_aboutAction);
	helpMenu->addAction(_qtAction);

	_menuBar->addMenu(fileMenu);
	_menuBar->addMenu(helpMenu);

	this->setMenuBar(_menuBar);
	this->setMinimumWidth(640);
	this->setMinimumHeight(480);

	_newButton = new QPushButton(tr("New"));
	_importButton = new QPushButton(tr("Import"));
	_sampleButton = new QPushButton(tr("Sample"));
	_tabWidget = new QTabWidget;
	_slider = new QSlider(Qt::Horizontal);
	_percentLabel = new QLabel;
	_coreWidget = new XCsvTableWidget;
	_graphWidget = new XGraphWidget;
	_csvWidget = new XCsvTableWidget;
	_srcWidget = new XCsvTableWidget;

	_slider->setRange(1, 100);
	_slider->setSingleStep(1);
	_slider->setPageStep(1);
	_slider->setTracking(false);

	this->connect(_newButton, SIGNAL(clicked()), this, SLOT(wizard()));
	this->connect(_importButton, SIGNAL(clicked()), this, SLOT(importWorkspace()));
	this->connect(_sampleButton, SIGNAL(clicked()), this, SLOT(sample()));
	this->connect(_slider, SIGNAL(sliderMoved(int)), _percentLabel, SLOT(setNum(int)));
	this->connect(_slider, SIGNAL(valueChanged(int)), _percentLabel, SLOT(setNum(int)));
	this->connect(_slider, SIGNAL(valueChanged(int)), this, SLOT(update(int)));

	QWidget *startWidget = new QWidget;
	QGridLayout *startLayout = new QGridLayout;

	startLayout->addWidget(new QLabel(tr("Open a file or create a new table")), 0, 0);
	startLayout->addWidget(_newButton, 0, 1);
	startLayout->addWidget(new QLabel(tr("Import an existing workspace")), 1, 0);
	startLayout->addWidget(_importButton, 1, 1);
	startLayout->addWidget(new QLabel(tr("Show an input file sample")), 2, 0);
	startLayout->addWidget(_sampleButton, 2, 1);
	startLayout->setAlignment(Qt::AlignCenter);

	startWidget->setLayout(startLayout);

	QWidget *resultWidget = new QWidget;
	QHBoxLayout *sliderLayout = new QHBoxLayout;

	sliderLayout->addWidget(_slider);
	sliderLayout->addWidget(_percentLabel);
	sliderLayout->addWidget(new QLabel("%"));

	QVBoxLayout *coreLayout = new QVBoxLayout;

	coreLayout->addLayout(sliderLayout);
	coreLayout->addWidget(_coreWidget);

	resultWidget->setLayout(coreLayout);

	_tabWidget->addTab(startWidget, tr("Start"));
	_tabWidget->addTab(resultWidget, tr("Core collection"));
	_tabWidget->addTab(_graphWidget, tr("Graph"));
	_tabWidget->addTab(_csvWidget, tr("Input file"));
	_tabWidget->addTab(_srcWidget, tr("Starting collection"));
	_tabWidget->setTabEnabled(1, false);
	_tabWidget->setTabEnabled(2, false);
	_tabWidget->setTabEnabled(3, false);
	_tabWidget->setTabEnabled(4, false);

	this->setCentralWidget(_tabWidget);
	this->setWindowIcon(QIcon(":/img/dna"));
	this->setWindowTitle(CC_NAME);
}

XMainWindow::~XMainWindow()
{
	delete _workspace;
}

void XMainWindow::clearWorkspace(XWorkspace *newWorkspace)
{
	if (_workspace)
	{
		_current = NULL;
		_coreWidget->setCsv(NULL);
		_csvWidget->setCsv(NULL);
		_srcWidget->setCsv(NULL);

		delete _workspace;

		_workspace = NULL;
	}

	if (newWorkspace)
	{
		_workspace = newWorkspace;

		_tabWidget->setTabEnabled(1, true);
		_tabWidget->setTabEnabled(2, true);
		_tabWidget->setTabEnabled(3, true);
		_tabWidget->setCurrentIndex(1);

		_graphWidget->setResult(newWorkspace->result());
		_csvWidget->setCsv(_workspace->csv());

		if (_workspace->src())
		{
			_srcWidget->setCsv(_workspace->src());
			_tabWidget->setTabEnabled(4, true);
		}
		else
		{
			_tabWidget->setTabEnabled(4, false);
		}

		int value = _workspace->slider();

		if (value < 1 || value > 100)
		{
			value = 100;
		}

		_slider->setValue(value);
		update(value);

		_exportAction->setEnabled(true);
		_saveAction->setEnabled(true);
		_saveAsAction->setEnabled(true);
	}
}

void XMainWindow::saveCurrent(const QString &path) const
{
	if (path.endsWith(".xls"))
	{
		BasicExcel xls;

		xls.New();

		BasicExcelWorksheet *sheet = xls.GetWorksheet((size_t)0);

		for (int i = 0; i < _current->rows(); ++i)
		{
			for (int j = 0; j < _current->columns(); ++j)
			{
				BasicExcelCell *cell = sheet->Cell(i, j);

				QVariant value = _current->at(i, j);
				bool ok;
				int integer = value.toInt(&ok);

				if (ok)
					cell->Set(integer);
				else
					cell->Set(value.toString().toLatin1().data());
			}
		}

		xls.SaveAs(path.toLatin1().data());
	}
	else
	{
		_current->save(path);
	}
}

void XMainWindow::sample()
{
	XSample example;

	centerResize(example);

	example.exec();
}

void XMainWindow::wizard()
{
	if (_csv)
		return;

	XWizard wizard(this);

	this->connect(&wizard, SIGNAL(finished(XCsv*,XCsv*,int,int)), this, SLOT(parseCsv(XCsv*,XCsv*,int,int)));

	wizard.exec();
}

void XMainWindow::parseCsv(XCsv *csv, XCsv *src, int iterations, int seed)
{
	if (_csv)
		return;

	_csv = csv;
	_src = src;

	XCore core(csv, src, iterations, seed);

	QProgressDialog progressDialog(tr("Working..."), tr("Abort"), 1, 100, this);

	this->connect(&core, SIGNAL(percentChanged(int)), &progressDialog, SLOT(setValue(int)));
	this->connect(&core, SIGNAL(error(QString)), &progressDialog, SLOT(close()));
	this->connect(&core, SIGNAL(finished()), &progressDialog, SLOT(close()));
	this->connect(&core, SIGNAL(completed(XResult *)), this, SLOT(csvParseSucceeded(XResult *)));
	this->connect(&core, SIGNAL(error(QString)), this, SLOT(csvParseFailed(QString)));
	this->connect(&progressDialog, SIGNAL(canceled()), this, SLOT(csvParseCanceled()));
	this->connect(&progressDialog, SIGNAL(canceled()), &core, SLOT(stopIterating()));

	core.start();
	progressDialog.exec();
	core.wait();
}

void XMainWindow::csvParseSucceeded(XResult *result)
{
	this->clearWorkspace(new XWorkspace(_csv, _src, result));

	_csv = NULL;
	_src = NULL;
}

void XMainWindow::csvParseFailed(QString error)
{
	delete _csv;
	delete _src;

	_csv = NULL;
	_src = NULL;

	QMessageBox::critical(this, tr("Error"), error, QMessageBox::Ok);
}

void XMainWindow::csvParseCanceled()
{
	delete _csv;
	delete _src;

	_csv = NULL;
	_src = NULL;
}

void XMainWindow::save()
{
	QString path = this->_workspace->savePath();

	path == "" ? this->saveAs() : this->saveCurrent(path);
}

void XMainWindow::saveAs()
{
	QString path = QFileDialog::getSaveFileName(this, "", "", CSV_FILTER + ";;" + EXCEL_FILTER);

	if (path.isEmpty())
		return;

	_workspace->setSavePath(path);
	this->saveCurrent(path);
}

void XMainWindow::importWorkspace()
{
	QString path = QFileDialog::getOpenFileName(this, "", "", WORKSPACE_FILTER);

	if (path.isEmpty())
		return;

	XWorkspace *workspace = new XWorkspace(path);

	if (workspace)
		this->clearWorkspace(workspace);
}

void XMainWindow::exportWorkspace()
{
	QString path = QFileDialog::getSaveFileName(this, "", "", WORKSPACE_FILTER);

	if (path.isEmpty())
		return;

	_workspace->save(path);
}

void XMainWindow::update(int value)
{
	if (!_workspace)
		return;

	XCsv *csv = _workspace->result()->csv(value);

	if (csv && csv != _current)
	{
		_current = csv;
		_coreWidget->setCsv(csv);
	}

	_workspace->setSlider(value);
}
