#include "xresult.h"

XResult::XResult(const QString &input, const XCsv *csv, const XCsv *src, QObject *parent) :
		XFile(parent)
{
	_csv = csv;
	_src = src;

	QString data = this->getData(input);

	if (data != "")
		this->fromString(data);
}

XResult::XResult(const QList<XResultEntry> &rows, const XCsv *csv, const XCsv *src, QObject *parent) :
		XFile(parent)
{
	_csv = csv;
	_src = src;
	_rows = rows;
}

XResult::~XResult()
{
	QList<XCsv *> deleted;

	foreach (XCsv *csv, _cache)
	{
		if (!deleted.contains(csv))
		{
			deleted << csv;

			delete csv;
		}
	}
}

int XResult::count(const int &percent) const
{
	return _rows[percent - 1].first;
}

XCsv *XResult::csv(const int &percent)
{
	if (!_cache.contains(percent))
	{
		int elements = _rows[percent - 1].second.length();

		for (int i = 0; i < _rows.length() && _rows[i].second.length() <= elements; ++i)
		{
			if (_rows[i].second == _rows[percent - 1].second)
			{
				if (_cache.contains(i + 1))
				{
					_cache[percent] = _cache[i + 1];
					return _cache[percent];
				}
			}
		}

		XCsvData data;

		if (_src != NULL)
			data << _src->data();
		else
			data << _csv->at(0);

		foreach (int row, _rows[percent - 1].second)
		{
			data << _csv->at(row);
		}

		_cache[percent] = new XCsv(data);
	}

	return _cache[percent];
}

QString XResult::toString() const
{
	QString str;
	QTextStream stream(&str);

	for (int i = 0; i < 100; ++i)
	{
		foreach (int value, _rows[i].second)
		{
			stream << value << ",";
		}

		stream << _rows[i].first << endl;
	}

	return str;
}

void XResult::fromString(QString &input)
{
	int count = 0;

	QTextStream stream(&input);

	while (!stream.atEnd())
	{
		QString line = stream.readLine().trimmed();
		QStringList split = line.split(',');

		if (line.isEmpty())
			continue;

		int kne = split.takeLast().toInt();
		XList row;

		foreach (QString value, split)
		{
			row << value.toInt();
		}

		_rows << XResultEntry(kne, row);
		++count;
	}
}
