#ifndef XWIZARD_H
#define XWIZARD_H

#include <QtGui>
#include <climits>
#include "../ExcelFormat/ExcelFormat.h"
#include "xcommon.h"
#include "xcsv.h"
#include "xcsveditor.h"

using namespace ExcelFormat;

class XWizard : public QWizard
{
	Q_OBJECT
public:
	enum PageType { PageFirst, PageCsvSelection, PageManualEditing, PageSettings };
	explicit XWizard(QWidget *parent = 0);

private slots:
	void finished(int result);

signals:
	void finished(XCsv *, XCsv *, int, int);
};

class XFirstPage : public QWizardPage
{
	Q_OBJECT
public:
	explicit XFirstPage(QWidget *parent = 0);

protected:
	int nextId() const;

private:
	QRadioButton *_fileInputRadioButton;
	QRadioButton *_manualInputRadioButton;
};

class XCsvSelectionPage : public QWizardPage
{
	Q_OBJECT
public:
	explicit XCsvSelectionPage(QWidget *parent = 0);
	XCsv *csv() const;
	XCsv *src() const;

protected:
	void initializePage();
	bool isComplete() const;
	int nextId() const;

private:
	XCsv *_csv;
	XCsv *_src;
	XCsvTableModel *_csvModel;
	XCsvTableModel *_srcModel;
	QLineEdit *_csvPathLineEdit;
	QPushButton *_csvBrowseButton;
	QLabel *_csvErrorLabel;
	QPushButton *_csvEditButton;
	QCheckBox *_srcCheckBox;
	QLineEdit *_srcPathLineEdit;
	QPushButton *_srcBrowseButton;
	QLabel *_srcErrorLabel;
	QPushButton *_srcEditButton;
	void switchState(QLabel *label, QPushButton *button, const bool &errors);
	void switchSrcSection(const bool &enable);
	XCsv *parseFile(const QString &path) const;

private slots:
	void csvBrowseButtonClicked();
	void csvEditButtonClicked();
	void srcCheckboxToggled(bool checked);
	void srcBrowseButtonClicked();
	void srcEditButtonClicked();
};

class XManualEditingPage : public QWizardPage
{
	Q_OBJECT
public:
	explicit XManualEditingPage(QWidget *parent = 0);
	XCsv *csv() const;

protected:
	void initializePage();
	bool isComplete() const;
	int nextId() const;

private:
	XCsv *_csv;
	XCsvTableModel *_csvModel;
	QLabel *_rowsLabel;
	QSpinBox *_rowsSpinBox;
	QLabel *_columnsLabel;
	QSpinBox *_columnsSpinBox;
	QLabel *_fillLabel;
	QLineEdit *_fillLineEdit;
	QLabel *_createLabel;
	QPushButton *_createButton;
	QLabel *_editLabel;
	QPushButton *_editButton;
	void switchState(const bool &errors);

private slots:
	void createButtonClicked();
	void editButtonClicked();
};

class XSettingsPage : public QWizardPage
{
	Q_OBJECT
public:
	explicit XSettingsPage(QWidget *parent = 0);

protected:
	void initializePage();
	int nextId() const;

private:
	QLabel *_iterationsLabel;
	QSpinBox *_iterationsSpinBox;
	QLabel *_seedLabel;
	QSpinBox *_seedSpinBox;
};

#endif // XWIZARD_H
