#include "xworkspace.h"

XWorkspace::XWorkspace(const QString &input, QObject *parent) :
		XFile(parent)
{
	QString data = this->getData(input);

	if (data != "")
		this->fromString(data);
}

XWorkspace::XWorkspace(XCsv *csv, XCsv *src, XResult *result, QObject *parent) :
		XFile(parent)
{
	_csv = csv;
	_src = src;
	_result = result;

	if (_src != NULL)
		_values[WKSPC_VALUE_SOURCE] = true;
}

XWorkspace::~XWorkspace()
{
	delete _csv;
	delete _src;
	delete _result;
}

XCsv *XWorkspace::csv() const
{
	return _csv;
}

XCsv *XWorkspace::src() const
{
	return _src;
}

XResult *XWorkspace::result() const
{
	return _result;
}

bool XWorkspace::hasSource() const
{
	return _values.contains(WKSPC_VALUE_SOURCE) && _values[WKSPC_VALUE_SOURCE].toBool() == true;
}

int XWorkspace::slider() const
{
	if (!_values.contains(WKSPC_VALUE_SLIDER))
		return -1;

	bool ok;
	int value = _values[WKSPC_VALUE_SLIDER].toInt(&ok);

	if (!ok)
		value =  _values[WKSPC_VALUE_SLIDER].toString().toInt(&ok);

	if (!ok)
		return -1;

	return value;
}

void XWorkspace::setSlider(const int &value)
{
	_values[WKSPC_VALUE_SLIDER] = value;
}

QString XWorkspace::savePath() const
{
	return _savePath;
}

void XWorkspace::setSavePath(const QString &path)
{
	_savePath = path;
}

QString XWorkspace::toString() const
{
	QString str;
	QTextStream stream(&str);

	foreach (QString key, _values.keys())
	{
		stream << key << "=" << _values[key].toString() << endl;
	}

	stream << endl << WKSPC_SEPARATOR;

	stream << endl << _csv->toString() << endl << "#";

	if (this->hasSource())
		stream << endl << _src->toString() << endl << "#";

	stream << endl << _result->toString() << endl << "#";

	return str;
}

void XWorkspace::fromString(QString &input)
{
	QTextStream stream(&input);

	int state = 0;

	while (!stream.atEnd())
	{
		QString line;

		if (line == WKSPC_SEPARATOR)
			++state;

		while ((line = stream.readLine()) != WKSPC_SEPARATOR)
		{
			if (!line.isEmpty())
			{
				QStringList split = line.split('=');

				_values.insert(split.at(0).trimmed(), split.at(1).trimmed());
			}
		}

		QString buffer;
		QTextStream bstream(&buffer);

		while ((line = stream.readLine()) != WKSPC_SEPARATOR)
		{
			bstream << line << endl;
		}

		_csv = new XCsv(buffer);

		buffer.clear();

		if (this->hasSource())
		{
			while ((line = stream.readLine()) != WKSPC_SEPARATOR)
			{
				bstream << line << endl;
			}

			_src = new XCsv(buffer);
		}
		else
		{
			_src = NULL;
		}

		buffer.clear();

		while ((line = stream.readLine()) != WKSPC_SEPARATOR)
		{
			bstream << line << endl;
		}

		_result = new XResult(buffer, _csv, _src);
	}
}
